BASE_URL_MAINNET = "https://mainnet.umi.top"
BASE_URL_TESTNET = "https://testnet.umi.top"


class Prefix:
    UMI = "umi"
    GLIZE = "glz"
    GLIZE_STAKING = "gls"

    ROD_STAKING = "rod01"
    ROD_TRADING = "rod"
